SystemOrganization addCategory: #MonticelloConfigurations!

MCVersionReader subclass: #MCMcmReader
	instanceVariableNames: 'fileName configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmReader class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:47'!
extension
	^ 'mcm'! !

!MCMcmReader class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
on: aStream fileName: aFileName
	| reader |
	reader := self on: aStream.
	reader fileName: aFileName.
	^reader! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 11/26/2005 20:26'!
configuration
	configuration ifNil: [self loadConfiguration].
	"browser modifies configuration, but the reader might get cached"
	^configuration copy! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 3/23/2005 01:17'!
configurationName
	^fileName ifNotNil: [(fileName findTokens: '/\:') last copyUpToLast: $.]! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 3/23/2005 01:17'!
fileName: aString
	fileName := aString! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 11/16/2005 11:03'!
loadConfiguration
	stream reset.
	configuration := MCConfiguration fromArray: (MCScanner scan: stream).
	configuration name: self configurationName.
! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 11/16/2005 11:01'!
loadVersionInfo
	info := self configuration! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 11/16/2005 11:01'!
version
	^self configuration! !

RWBinaryOrTextStream subclass: #MCPseudoFileStream
	instanceVariableNames: 'localName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCPseudoFileStream commentStamp: '<historical>' prior: 0!
A pseudo file stream which can be used for updates.!

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName
	^localName! !

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName: aString
	localName := aString! !

Object subclass: #MCConfiguration
	instanceVariableNames: 'name dependencies repositories log'
	classVariableNames: 'DefaultLog UpgradeIsMerge'
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCConfiguration commentStamp: 'StephaneDucasse 11/29/2011 22:23' prior: 0!
An MCConfiguration specifies the configuration of a set of related Monticello packages. It maintains an ordered list of package versions and a list of repositories in which the packages may be found.

An MCConfiguration may be filed out for storage as an array specification, and new instances can be created from a stored array specification.
!

!MCConfiguration class methodsFor: 'utilities' stamp: 'bf 9/6/2006 19:39'!
changesIn: aPackage from: oldInfo to: newInfo on: aStream
	| printBlock newVersion |
	(newInfo = oldInfo)
		ifTrue: [^self].
	aStream cr; nextPutAll: '----------------- ', aPackage name, ' ------------------'; cr.

	newInfo 
		ifNil: [^aStream cr; nextPutAll: 'REMOVED'; cr].

	oldInfo 
		ifNil: [^aStream cr; nextPutAll: 'ADDED'; cr].

	"get actual version for full ancestry"
	newVersion := MCRepositoryGroup default versionWithInfo: newInfo.

	printBlock := [:ver | 
		aStream cr; 
			nextPutAll: (ver name copyAfterLast: $-);
			nextPutAll:  ' (', (ver date printFormat: #(1 2 0 $. 1 1 2)), ', '.
		ver time print24: true showSeconds: false on: aStream.
		aStream nextPutAll: ')'; cr; 
			nextPutAll: ver message; cr].

	(newVersion info hasAncestor: oldInfo)
		ifTrue: [(newVersion info allAncestorsOnPathTo: oldInfo)
			reverseDo: printBlock].
	newVersion info in: printBlock.
	aStream flush! !

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog
	"Answer the default configuration log"
	^DefaultLog! !

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog: aStream
	"Set the default configuration log"
	DefaultLog := aStream.! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:43'!
dependencyFromArray: anArray
	^MCVersionDependency
		package: (MCPackage named: anArray first)
		info: (
			MCVersionInfo
			name: anArray second
			id: (UUID fromString: anArray third)
			message: nil
			date: nil
			time: nil
			author: nil
			ancestors: nil)! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:44'!
dependencyToArray: aDependency
	^ {
		aDependency package name . 
		aDependency versionInfo name . 
		aDependency versionInfo id asString }! !

!MCConfiguration class methodsFor: 'instance creation' stamp: 'bf 3/24/2005 01:51'!
fromArray: anArray
	| configuration |
	configuration := self new.
	anArray pairsDo: [:key :value |
		key = #repository
			ifTrue: [configuration repositories add: (self repositoryFromArray: value)].
		key = #dependency
			ifTrue: [configuration dependencies add: (self dependencyFromArray: value)].
	].
	^configuration! !

!MCConfiguration class methodsFor: 'converting' stamp: 'SeanDeNigris 8/26/2012 11:35'!
repositoryFromArray: anArray
	^ MCRepositoryGroup default repositories
		detect: [:repo | repo description = anArray first]
		ifNone: [MCHttpRepository location: anArray first]! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:51'!
repositoryToArray: aRepository
	^ {aRepository description}! !

!MCConfiguration class methodsFor: 'settings' stamp: 'AlainPlantec 12/17/2009 22:58'!
upgradeIsMerge
	^ UpgradeIsMerge ifNil: [UpgradeIsMerge := false]! !

!MCConfiguration class methodsFor: 'settings' stamp: 'AlainPlantec 12/17/2009 22:58'!
upgradeIsMerge: aBoolean
	UpgradeIsMerge := aBoolean! !

!MCConfiguration class methodsFor: 'utilities' stamp: 'bf 6/13/2006 15:46'!
whatChangedFrom: oldConfig to: newConfig
	"MCConfiguration
		whatChangedFrom: ReleaseBuilderPloppDeveloper config20060201PloppBeta
		to:  ReleaseBuilderPloppDeveloper config20060215premaster" 

	self whatChangedFrom: oldConfig to: newConfig on: Transcript.
	Transcript flush.! !

!MCConfiguration class methodsFor: 'utilities' stamp: 'bf 6/13/2006 15:47'!
whatChangedFrom: oldConfig to: newConfig on: aStream
	"MCConfiguration
		whatChangedFrom:  ReleaseBuilderPloppDeveloper config20060201PloppBeta
		to:  ReleaseBuilderPloppDeveloper config20060215premaster" 

	| oldDeps |
	oldDeps := Dictionary new.
	oldConfig dependencies do: [:old | oldDeps at: old package put: old].

	newConfig dependencies do: [:new | | old |
		old := oldDeps removeKey: new package ifAbsent: [nil].
		old ifNotNil: [old := old versionInfo].
		self changesIn: new package from: old to: new versionInfo on: aStream.
	].

	oldDeps do: [:old |
		self changesIn: old package from: old versionInfo to: nil on: aStream.
	].
! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:19'!
changes
	^MCPatch operations: #()! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:32'!
dependencies
	^dependencies ifNil: [dependencies := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 18:40'!
dependencies: aCollection
	dependencies := aCollection! !

!MCConfiguration methodsFor: 'private' stamp: 'StephaneDucasse 2/20/2010 23:07'!
depsSatisfying: selectBlock versionDo: verBlock displayingProgress: progressString
	| repoMap count |
	repoMap := Dictionary new.
	self repositories do: [:repo |
		MCRepositoryGroup default addRepository: repo.
		repo allVersionNames
			ifEmpty: [self logWarning: 'cannot read from ', repo description]
			ifNotEmptyDo: [:all | all do: [:ver | repoMap at: ver put: repo]]].

	count := 0.
	self dependencies do: [:dep |
		| ver repo |
		ver := dep versionInfo name.
		repo := repoMap 
					at: ver 
					ifAbsent: 
						[self logError: 'Version ', ver, ' not found in any repository'.
						self logError: 'Aborting'.
						^count].
		(selectBlock value: dep) 
			ifTrue: [| new |
					new := self versionNamed: ver for: dep from: repo.
					new ifNil: 
							[self logError: 'Could not download version ', ver, ' from ', repo description.
							self logError: 'Aborting'.
							^count]
						ifNotNil: 
							[ self logUpdate: dep package with: new.
							ProgressNotification signal: '' extra: 'Installing ', ver.
							verBlock value: new.
							count := count + 1]].
		dep package workingCopy repositoryGroup addRepository: repo.
	] displayingProgress: progressString.
	^count! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 11:26'!
diffBaseFor: aDependency
	| wc |
	aDependency package hasWorkingCopy ifFalse: [^nil].
	wc := aDependency package workingCopy.
	wc ancestors ifEmpty: [^nil].
	^wc ancestors first name! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:22'!
fileName
	^ self name, '.', self writerClass extension
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/22/2005 10:51'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:17'!
info
	^MCVersionInfo new! !

!MCConfiguration methodsFor: 'initialize' stamp: 'ar 5/27/2005 17:28'!
initialize
	super initialize.
	log := DefaultLog.! !

!MCConfiguration methodsFor: 'testing' stamp: 'bf 3/22/2005 22:56'!
isCacheable
	^false! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/17/2006 15:55'!
load
	"install all the versions in this configuration, even if this means to downgrade a package"

	| versions |
	versions := OrderedCollection new.

	self depsSatisfying: [:dep | dep isCurrent not]
		versionDo: [:ver | versions add: ver]
		displayingProgress: 'finding packages'.

	^self loadVersions: versions! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 3/16/2006 19:07'!
loadVersions: aCollection

	| loader |
	aCollection isEmpty ifTrue: [^0].
	loader := MCVersionLoader new.
	aCollection do: [:each | loader addVersion: each].
	loader loadWithNameLike: self nameForChangeset.
	^ aCollection size! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 6/9/2005 15:58'!
log
	^log ifNil: [Transcript]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'ar 4/28/2005 11:55'!
log: aStream
	log := aStream.! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:07'!
logError: aString
	self log
		cr; nextPutAll: 'ERROR: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 15:59'!
logUpdate: aPackage with: aVersion
	self log
		cr; nextPutAll: '========== ', aVersion info name, ' =========='; cr;
		cr; nextPutAll: aVersion info message asString; cr;
		flush.

	aPackage hasWorkingCopy ifFalse: [^self].

	aPackage workingCopy ancestors do: [:each |
		(aVersion info hasAncestor: each)
			ifTrue: [(aVersion info allAncestorsOnPathTo: each)
				do: [:ver | self log cr; nextPutAll: '>>> ', ver name, ' <<<'; cr;
							nextPutAll: ver message; cr; flush]]]! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:08'!
logWarning: aString
	self log
		cr; nextPutAll: 'WARNING: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/16/2006 17:41'!
merge
	"merge in all the versions in this configuration"

	| versions |
	versions := OrderedCollection new.

	self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | versions add: ver]
		displayingProgress: 'finding packages'.

	^self mergeVersions: versions! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 3/16/2006 19:07'!
mergeVersions: aCollection

	| merger |
	aCollection isEmpty ifTrue: [^0].
	merger := MCVersionMerger new.
	aCollection do: [:each | merger addVersion: each].
	merger mergeWithNameLike: self nameForChangeset.
	^ aCollection size! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 3/20/2006 19:10'!
mergeVersionsSilently: aCollection

	^self suppressMergeDialogWhile: [self mergeVersions: aCollection]! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 5/23/2005 14:47'!
mustMerge: aVersion
	"answer true if we have to do a full merge and false if we can simply load instead"
	
	| pkg wc current |
	(pkg := aVersion package) hasWorkingCopy ifFalse: [^false "no wc -> load"].
	(wc := pkg workingCopy) modified ifTrue: [^true "modified -> merge"].
	wc ancestors isEmpty ifTrue: [^true "no ancestor info -> merge"].
	current := wc ancestors first.
	(aVersion info hasAncestor: current) ifTrue: [^false "direct descendant of wc -> load"].
	"new branch -> merge"
	^true! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name
	^name! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name: aString
	name := aString! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 3/16/2006 19:07'!
nameForChangeset
	^self name ifNil: [self class name]! !

!MCConfiguration methodsFor: 'copying' stamp: 'bf 11/26/2005 20:22'!
postCopy
	dependencies := dependencies shallowCopy.
	repositories := repositories shallowCopy.! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:35'!
repositories
	^repositories ifNil: [repositories := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:36'!
repositories: aCollection
	repositories := aCollection! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 00:44'!
summary
	^String streamContents: [:stream |
		self dependencies
			do: [:ea | stream nextPutAll: ea versionInfo name; cr ]]! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 3/20/2006 19:09'!
suppressMergeDialogWhile: aBlock
	^[aBlock value]
		on: MCMergeResolutionRequest
		do: [:request |
			request merger conflicts isEmpty
				ifTrue: [request resume: true]
				ifFalse: [request pass]]! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 5/23/2005 17:43'!
updateFromImage
	self dependencies: (self dependencies collect: [:dep |
		dep package hasWorkingCopy
			ifTrue: [
				dep package workingCopy in: [:wc |
					MCVersionDependency package: wc package info: wc ancestors first]]
			ifFalse: [dep]]).
! !

!MCConfiguration methodsFor: 'updating' stamp: 'StephaneDucasse 2/20/2010 23:07'!
updateFromRepositories
	| oldInfos newNames sortedNames newDeps |
	oldInfos := self dependencies collect: [:dep | dep versionInfo].
	newNames := Dictionary new.
	self repositories
		do: [ :repo | 
			ProgressNotification signal: '' extra: 'Checking ', repo description.
			(repo possiblyNewerVersionsOfAnyOf: oldInfos)
				do: [:newName | newNames at: newName put: repo]]
		displayingProgress: 'Searching new versions'.

	sortedNames := newNames keys asSortedCollection:
		[:a :b | a numericSuffix > b numericSuffix].

	newDeps := OrderedCollection new.
	self dependencies do: [:dep |
		| newName |
		newName := sortedNames
			detect: [:each | (each copyUpToLast: $-) = dep package name]
			ifNone: [nil].
		newDeps add: (newName
			ifNil: [dep]
			ifNotNil: [
				| repo ver  |
				repo := newNames at: newName.
				ver := self versionNamed: newName for: dep from: repo.
				ver ifNil: [dep]
					ifNotNil: [MCVersionDependency package: ver package info: ver info]
			])
	] displayingProgress: 'downloading new versions'.

	self dependencies: newDeps.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'AlainPlantec 12/17/2009 22:59'!
upgrade
	^self class upgradeIsMerge
		ifTrue: [self upgradeByMerging]
		ifFalse: [self upgradeByLoading]! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/17/2006 16:17'!
upgradeByLoading
	"this differs from #load only in that newer versions in the image are not downgraded"

	| versions |
	versions := OrderedCollection new.

	self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | versions add: ver]
		displayingProgress: 'finding packages'.

	^self loadVersions: versions
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/20/2006 19:10'!
upgradeByMerging
	| versions |
	versions := OrderedCollection new.

	self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | versions add: ver]
		displayingProgress: 'finding packages'.

	^(versions noneSatisfy: [:ver | self mustMerge: ver])
		ifTrue: [self loadVersions: versions]
		ifFalse: [self mergeVersionsSilently: versions].
! !

!MCConfiguration methodsFor: 'private' stamp: 'md 9/18/2005 15:56'!
versionNamed: verName for: aDependency from: repo

	| baseName fileName ver |
	(repo filterFileNames: repo cachedFileNames forVersionNamed: verName) ifNotEmptyDo: [:cachedNames |
		fileName := cachedNames anyOne.
		ProgressNotification signal: '' extra: 'Using cached ', fileName.
		ver := repo versionFromFileNamed: fileName].
	ver ifNil: [
		baseName := self diffBaseFor: aDependency.
		(baseName notNil and: [baseName ~= verName and: [repo includesVersionNamed: baseName]]) ifTrue: [
			fileName := (MCDiffyVersion nameForVer: verName base: baseName), '.mcd'.
			ProgressNotification signal: '' extra: 'Downloading ', fileName.
			ver := repo versionFromFileNamed: fileName]].
	ver ifNil: [
		fileName := verName, '.mcz'.
		ProgressNotification signal: '' extra: 'Downloading ', fileName.
		ver := repo versionFromFileNamed: fileName].
	^ver! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:50'!
writerClass
	^ MCMcmWriter ! !

MCWriter subclass: #MCMcmWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmWriter class methodsFor: 'writing' stamp: 'stephaneducasse 2/4/2006 20:47'!
fileOut: aConfiguration on: aStream
	| inst |
	inst := self on: aStream.
	inst writeConfiguration: aConfiguration.
	inst close.
	
! !

!MCMcmWriter class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:49'!
readerClass
	^ MCMcmReader! !

!MCMcmWriter methodsFor: 'writing' stamp: 'bf 3/22/2005 18:00'!
close
	stream close! !

!MCMcmWriter methodsFor: 'writing' stamp: 'bf 3/24/2005 01:50'!
writeConfiguration: aConfiguration

	stream nextPut: $(.

	aConfiguration repositories do: [:ea | 
		stream cr.
		stream nextPutAll: 'repository '.
		(MCConfiguration repositoryToArray: ea) printElementsOn: stream].

	aConfiguration dependencies do: [:ea | 
		stream cr.
		stream nextPutAll: 'dependency '.
		(MCConfiguration dependencyToArray: ea) printElementsOn: stream].

	stream cr.
	stream nextPut: $).
	stream cr.! !
